/*
   AtrSIOHandler.cpp - handles SIO commands, the image data is stored
   in an AtrImage object

   Copyright (C) 2002, 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>
#include <stdio.h>
#include "AtrSIOHandler.h"
#include "AtariDebug.h"
#include "HighSpeedSIOCode.h"

/*
 * speed definition bytes for the Atari:
 * the default for ~19200 bit/sec is 40.
 * ~57600 bit/sec uses a value of 8.
 */

#define SPEED_BYTE_19200 40
#define SPEED_BYTE_38400 16
#define SPEED_BYTE_57600 8

AtrSIOHandler::AtrSIOHandler(const RCPtr<AtrImage>& image)
	: fImage(image),
	  fEnableHighSpeed(false),
	  fSpeedByte(SPEED_BYTE_57600),
	  fLastFDCStatus(0xff)
{
	if (fImage) {
		fImageConfig = fImage->GetImageConfig();
		fFormatConfig = fImageConfig;
	}
	fTracer = SIOTracer::GetInstance();
}

AtrSIOHandler::~AtrSIOHandler()
{
}

int AtrSIOHandler::ProcessCommandFrame(SIO_command_frame& frame, const RCPtr<SIOWrapper>& wrapper)
{
	int ret=0, ret2;

	if (!fImage) {
		DPRINTF("error - no image loaded into AtrSIOHandler!");
		return AbstractSIOHandler::eImageError;
	}

	fTracer->TraceCommandFrame(frame);

	unsigned int myDriveNo = frame.device_id - 0x30;

	switch (frame.command) {
	case 0x53: {
		/* get status */

		if ((ret=wrapper->SendCommandACK())) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = 4;
		unsigned char buf[buflen];
		const char* description = "[ get status ]";

		buf[0] = 0x10; // motor on
		if (fFormatConfig.fSectorLength == e128BytesPerSector) {
			if (fFormatConfig.fNumberOfSectors==1040) {
				buf[0] |= 0x80; /* enhanced density */
			}
		} else {
			buf[0] |= 0x20; /* double density */
		}
		buf[1] = fLastFDCStatus;
		buf[2] = 0xe0;
		buf[3] = 0;

		if (fImage->IsWriteProtected()) {
			buf[0] |= 0x08;
		}

		fTracer->TraceCommandOK();
		fTracer->TraceGetStatus(myDriveNo);
		fTracer->TraceDataBlock(buf, 4, description);

		if ((ret=wrapper->SendComplete())) {
			DPRINTF("send complete failed");
			break;
		}

		if ((ret=wrapper->SendDataFrame(buf,4))) {
			DPRINTF("send data frame failed");
			break;
		}

		break;
	}
	case 0x52: {
		/* read sector */
		size_t sec = frame.aux1 + (frame.aux2<<8);
		if ( sec<=0 || sec > fImageConfig.fNumberOfSectors) {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}

			ret = AbstractSIOHandler::eIllegalSectorNumber;

			fTracer->TraceCommandError(ret);
			DPRINTF("illegal sector number %d",sec);

			break;
		}

		if ((ret=wrapper->SendCommandACK())) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = fImageConfig.GetSectorLength(sec);
		unsigned char buf[buflen];
		const char* description = "[ read sector ]";

		if (!fImage->ReadSector(sec, buf, buflen)) {
			fLastFDCStatus = 0xef; // record not found;
			ret = AbstractSIOHandler::eImageError;

			fTracer->TraceCommandError(ret);
			fTracer->TraceReadSector(myDriveNo, sec);
			fTracer->TraceDataBlock(buf, buflen, description);

			if (wrapper->SendError()) {
				DPRINTF("send error failed");
				break;
			}
		} else {
			fLastFDCStatus = 0xff;

			fTracer->TraceCommandOK();
			fTracer->TraceReadSector(myDriveNo, sec);
			fTracer->TraceDataBlock(buf, buflen, description);

			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}
		}

		if ((ret2=wrapper->SendDataFrame(buf, buflen))) {
			DPRINTF("send data frame failed");
			if (ret==0) ret=ret2;
			break;
		}
		break;
	}
	case 0x50:
	case 0x57: {
		/* write sector */
		size_t sec = frame.aux1 + (frame.aux2<<8);
		if ( sec<=0 || sec > fImageConfig.fNumberOfSectors ) {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}
			
			ret = AbstractSIOHandler::eIllegalSectorNumber;

			fTracer->TraceCommandError(ret);
			DPRINTF("illegal sector number %d",sec);
			
			break;
		}

		if ((ret=wrapper->SendCommandACK())) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = fImageConfig.GetSectorLength(sec);
		unsigned char buf[buflen];

		const char* description;
		if (frame.command == 0x50) {
			description = "[ put sector ]";
		} else {
			description = "[ write sector ]";
		}

		if ((ret=wrapper->ReceiveDataFrame(buf, buflen))) {
			fTracer->TraceCommandError(ret);
			DPRINTF("receive data frame failed");
			break;
		}

		bool lastChanged = fImage->Changed();

		if (fImage->IsWriteProtected() ) {
			fLastFDCStatus = 0xbf; // write protected
			ret = AbstractSIOHandler::eWriteProtected;

			fTracer->TraceCommandError(ret);
			if (frame.command == 0x50) {
				fTracer->TraceWriteSector(myDriveNo, sec);
			} else {
				fTracer->TraceWriteSectorVerify(myDriveNo, sec);
			}
			fTracer->TraceDataBlock(buf, buflen, description);

			if (wrapper->SendError()) {
				DPRINTF("send complete failed");
				break;
			}
			break;
		} else {
			if (!fImage->WriteSector(sec, buf, buflen)) {
				fLastFDCStatus = 0xb0; // write protected
				ret = AbstractSIOHandler::eImageError;

				fTracer->TraceCommandError(ret);
				if (frame.command == 0x50) {
					fTracer->TraceWriteSector(myDriveNo, sec);
				} else {
					fTracer->TraceWriteSectorVerify(myDriveNo, sec);
				}
				fTracer->TraceDataBlock(buf, buflen, description);

				if (wrapper->SendError()) {
					DPRINTF("send error failed");
					break;
				}
				break;
			} else {
				fLastFDCStatus = 0xff;

				if ((lastChanged == false) && fImage->Changed()) {
					fTracer->IndicateDriveChanged(myDriveNo);
				}
				fTracer->TraceCommandOK();
				if (frame.command == 0x50) {
					fTracer->TraceWriteSector(myDriveNo, sec);
				} else {
					fTracer->TraceWriteSectorVerify(myDriveNo, sec);
				}
				fTracer->TraceDataBlock(buf, buflen, description);

				if ((ret=wrapper->SendComplete())) {
					DPRINTF("send complete failed");
					break;
				}
			}
		}
		break;
	}
	case 0x4e: {
		/* percom get */

		if ((ret=wrapper->SendCommandACK()) ) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = 12;
		unsigned char buf[buflen];
		const char* description = "[ percom get ]";

		buf[0] = fFormatConfig.fTracksPerSide;
		buf[1] = 0;
		buf[2] = fFormatConfig.fSectorsPerTrack >> 8;
		buf[3] = fFormatConfig.fSectorsPerTrack & 0xff;
		if (fFormatConfig.fSides) {
			buf[4] = fFormatConfig.fSides - 1;
		} else {
			buf[4] = 0;
		}
		if (fFormatConfig.fDiskFormat == e90kDisk) {
			buf[5] = 0;
		} else {
			buf[5] = 4;
		}
		if (fFormatConfig.fSectorLength == e256BytesPerSector) {
			buf[6] = 1;
			buf[7] = 0;
		} else {
			buf[6] = 0;
			buf[7] = 128;
		}
		buf[8] = 1;
		buf[9] = 1;
		buf[10] = 0;
		buf[11] = 0;

		fTracer->TraceCommandOK();
		fTracer->TraceDecodedPercomBlock(myDriveNo, buf, true);
		fTracer->TraceDataBlock(buf, buflen, description);

		if ((ret=wrapper->SendComplete())) {
			DPRINTF("send complete failed");
			break;
		}

		if ((ret=wrapper->SendDataFrame(buf, 12))) {
			DPRINTF("send data frame failed");
			break;
		}

		break;
	}
	case 0x4f: {
		/* percom put */

		if ((ret=wrapper->SendCommandACK()) ) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = 12;
		unsigned char buf[buflen];
		const char* description = "[ percom put ]";

		if ((ret=wrapper->ReceiveDataFrame(buf, 12)) ) {
			fTracer->TraceCommandError(ret);
			DPRINTF("receive data frame failed");
			break;
		}

		size_t tracks, sec, sides, total, secLen;

		tracks = buf[0];
		sec = buf[3] + (buf[2] << 8);
		sides = buf[4] + 1;
		secLen = buf[7] + (buf[6]<<8);

		total = tracks * sec * sides;
		if (total > 0 && total < 65536 &&
		    (secLen == 128 || secLen == 256) ) {
			fFormatConfig.fTracksPerSide = tracks;
			fFormatConfig.fSectorsPerTrack = sec;
			fFormatConfig.fSides = sides;
			if (secLen == 256) {
				fFormatConfig.fSectorLength = e256BytesPerSector;
			} else {
				fFormatConfig.fSectorLength = e128BytesPerSector;
			}

			fFormatConfig.fNumberOfSectors = total;
			fFormatConfig.DetermineDiskFormatFromLayout();

/*
			int dsk = frame.device_id-0x30;
			switch (fFormatConfig.fDiskFormat) {
			case e90kDisk: DPRINTF("D%d: configured for 90k disk", dsk); break;
			case e130kDisk: DPRINTF("D%d: configured for 130k disk", dsk); break;
			case e180kDisk: DPRINTF("D%d: configured for 180k disk", dsk); break;
			case e360kDisk: DPRINTF("D%d: configured for 360k disk", dsk); break;
			default:
				DPRINTF("D%d: configured for %d sectors / %d bytes per sector",
					dsk, fFormatConfig.fNumberOfSectors,
					(int) fFormatConfig.fSectorLength);
				break;
			}
*/

			fTracer->TraceCommandOK();
			fTracer->TraceDecodedPercomBlock(myDriveNo, buf, false);
			fTracer->TraceDataBlock(buf, buflen, description);

			if ((ret=wrapper->SendComplete()) ) {
				DPRINTF("send complete failed");
				break;
			}
		} else {
			ret = AbstractSIOHandler::eInvalidPercomConfig;

			fTracer->TraceCommandError(ret);
			DPRINTF("invalid config: %d sectors, %d bytes per sector",total,secLen);
			fTracer->TraceDecodedPercomBlock(myDriveNo, buf, false);
			fTracer->TraceDataBlock(buf, buflen, description);

			if (wrapper->SendError()) {
				DPRINTF("send error failed");
				break;
			}
		}

		break;

	}

	case 0x20: {
		// format disk automatically (speedy 1050)
		//DPRINTF("format automatic (0x20)");

		if (!fEnableHighSpeed) {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}

			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);

			break;
		}
		if ((ret=wrapper->SendCommandACK()) ) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		if ( fImage->IsWriteProtected()) {
			fLastFDCStatus = 0xbf; // write protected
			ret = AbstractSIOHandler::eWriteProtected;
			fTracer->TraceCommandError(ret);
			if (wrapper->SendError()) {
				DPRINTF("send error failed");
				break;
			}
			break;
		}

		bool lastChanged = fImage->Changed();

		if (!fImage->CreateImage(fFormatConfig.fSectorLength,
					fFormatConfig.fSectorsPerTrack,
					fFormatConfig.fTracksPerSide,
					fFormatConfig.fSides)) {
			fLastFDCStatus = 0xbf; // write protected
			ret = AbstractSIOHandler::eImageError;
			fTracer->TraceCommandError(ret);
			DPRINTF("create image failed [%d %d]",
				fFormatConfig.fSectorLength, fFormatConfig.fNumberOfSectors);
			if (wrapper->SendError()) {
				DPRINTF("send error failed");
				break;
			}
			break;
		} else {
			fLastFDCStatus = 0xff;
			fImageConfig = fImage->GetImageConfig();
			fFormatConfig = fImageConfig;

			if ((lastChanged == false) && fImage->Changed()) {
				fTracer->IndicateDriveChanged(myDriveNo);
			}
			fTracer->IndicateDriveFormatted(myDriveNo);
			fTracer->TraceCommandOK();

			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}
		}

		break;
	}
	case 0x21: {
		// format disk

		if ((ret=wrapper->SendCommandACK())) {
			fTracer->TraceCommandError(ret);
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = fFormatConfig.fSectorLength;
		unsigned char buf[buflen];
		const char* description = "[ format disk ]";

		if (fImage->IsWriteProtected()) {
			fLastFDCStatus = 0xbf; // write protected
			ret = AbstractSIOHandler::eWriteProtected;
			memset(buf,0,buflen);

			fTracer->TraceCommandError(ret);
			fTracer->TraceFormatDisk(myDriveNo);
			fTracer->TraceDataBlock(buf, buflen, description);

			if (wrapper->SendError()) {
				DPRINTF("send error failed");
				break;
			}
		} else {
			bool lastChanged = fImage->Changed();

			if (!fImage->CreateImage(fFormatConfig.fSectorLength,
						fFormatConfig.fSectorsPerTrack,
						fFormatConfig.fTracksPerSide,
						fFormatConfig.fSides)) {
				fLastFDCStatus = 0xbf; // write protected
				ret = AbstractSIOHandler::eImageError;
				memset(buf,0,buflen);

				fTracer->TraceCommandError(ret);
				fTracer->TraceFormatDisk(myDriveNo);
				fTracer->TraceDataBlock(buf, buflen, description);

				DPRINTF("create image failed [%d %d]",
					fFormatConfig.fSectorLength, fFormatConfig.fNumberOfSectors);
				if (wrapper->SendError()) {
					DPRINTF("send error failed");
					break;
				}
			} else {
				fLastFDCStatus = 0xff;
				fImageConfig = fImage->GetImageConfig();
				fFormatConfig = fImageConfig;
				memset(buf,255,buflen);

				if ((lastChanged == false) && fImage->Changed()) {
					fTracer->IndicateDriveChanged(myDriveNo);
				}
				fTracer->IndicateDriveFormatted(myDriveNo);

				fTracer->TraceCommandOK();
				fTracer->TraceFormatDisk(myDriveNo);
				fTracer->TraceDataBlock(buf, buflen, description);

				if ((ret=wrapper->SendComplete())) {
					DPRINTF("send complete failed");
					break;
				}
			}
		}

		if ((ret2=wrapper->SendDataFrame(buf, buflen))) {
			DPRINTF("send data frame failed");
			if (ret==0) ret=ret2;
			break;
		}

		break;
	}

	case 0x22: {
		// format disk in enhanced density
		//DPRINTF("format enhanced (0x22)");

		if ((ret=wrapper->SendCommandACK())) {
			DPRINTF("send command ACK failed");
			break;
		}

		size_t buflen = 128;
		unsigned char buf[buflen];
		const char* description = "[ format enhanced density ]";

		if ( fImage->IsWriteProtected()) {
			fLastFDCStatus = 0xbf; // write protected
			ret = AbstractSIOHandler::eWriteProtected;
			memset(buf,0,128);

			fTracer->TraceCommandError(ret);
			fTracer->TraceFormatEnhanced(myDriveNo);
			fTracer->TraceDataBlock(buf, buflen, description);

			if (wrapper->SendError()) {
				DPRINTF("send error failed");
				break;
			}
		} else {
			bool lastChanged = fImage->Changed();

			if (!fImage->CreateImage(e130kDisk)) {
				fLastFDCStatus = 0xbf; // write protected
				ret = AbstractSIOHandler::eImageError;
				memset(buf,0,128);

				fTracer->TraceCommandError(ret);
				fTracer->TraceFormatEnhanced(myDriveNo);
				fTracer->TraceDataBlock(buf, buflen, description);

				if (wrapper->SendError()) {
					DPRINTF("send error failed");
					break;
				}
			} else {
				fLastFDCStatus = 0xff;
				fImageConfig = fImage->GetImageConfig();
				fFormatConfig = fImageConfig;
				memset(buf,255,128);

				if ((lastChanged == false) && fImage->Changed()) {
					fTracer->IndicateDriveChanged(myDriveNo);
				}
				fTracer->IndicateDriveFormatted(myDriveNo);

				fTracer->TraceCommandOK();
				fTracer->TraceFormatEnhanced(myDriveNo);
				fTracer->TraceDataBlock(buf, buflen, description);

				if ((ret=wrapper->SendComplete())) {
					DPRINTF("send complete failed");
					break;
				}
			}
		}

		if ((ret2=wrapper->SendDataFrame(buf, 128))) {
			DPRINTF("send data frame failed");
			if (ret==0) ret=ret2;
			break;
		}

		break;
	}

	case 0x3f: {
		// get ultra speed byte

		if (fEnableHighSpeed) {

			size_t buflen = 1;
			unsigned char buf[buflen];
			const char* description = "[ get speed byte ]";

			buf[0] = fSpeedByte;

			if ((ret=wrapper->SendCommandACK())) {
				fTracer->TraceCommandError(ret);
				DPRINTF("send command ACK failed");
				break;
			}

			fTracer->TraceCommandOK();
			fTracer->TraceGetSpeedByte(myDriveNo);
			fTracer->TraceDataBlock(buf, buflen, description);
	
			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}

			if ((ret=wrapper->SendDataFrame(buf, 1))) {
				DPRINTF("send data frame failed");
				break;
			}

		} else {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}

			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);
		}

		break;
	}
	case 0x68: {
		// get SIO length

		if (fEnableHighSpeed) {

			size_t buflen = 2;
			unsigned char buf[buflen];
			const char * description = "[ get SIO length ]";

			if ((ret=wrapper->SendCommandACK())) {
				fTracer->TraceCommandError(ret);
				DPRINTF("send command ACK failed");
				break;
			}
	
			size_t codelen = HighSpeedSIOCode::GetInstance()->GetCodeSize();

			buf[0] = codelen & 0xff;
			buf[1] = codelen >> 8;

			fTracer->TraceCommandOK(),
			fTracer->TraceGetSioCodeLength(myDriveNo);
			fTracer->TraceDataBlock(buf, buflen, description);

			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}

			if ((ret=wrapper->SendDataFrame(buf, 2))) {
				DPRINTF("send data frame failed");
				break;
			}

		} else {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}
			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);
		}

		break;
	}
	case 0x69: {
		// get SIO code

		if (fEnableHighSpeed) {

			size_t buflen = HighSpeedSIOCode::GetInstance()->GetCodeSize();
			unsigned char buf[buflen];
			const char* description = "[ get SIO code ]";

			if ((ret=wrapper->SendCommandACK())) {
				fTracer->TraceCommandError(ret);
				DPRINTF("send command ACK failed");
				break;
			}
	
			size_t relocadr = frame.aux1 + (frame.aux2<<8);

			HighSpeedSIOCode::GetInstance()->RelocateCode(buf, relocadr);

			fTracer->TraceCommandOK();
			fTracer->TraceGetSioCode(myDriveNo);
			fTracer->TraceDataBlock(buf, buflen, description);

			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}

			if ((ret=wrapper->SendDataFrame(buf, buflen))) {
				DPRINTF("send data frame failed");
				break;
			}

		} else {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}
			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);
		}

		break;
	}
	case 0x44: // configure drive
	case 0x4b: // slow/fast config
	case 0x51: // flush disks
	{
		// just ack these commands, but ignore them.

		if (fEnableHighSpeed) {

			if ((ret=wrapper->SendCommandACK())) {
				fTracer->TraceCommandError(ret);
				DPRINTF("send command ACK failed");
				break;
			}
	
			fTracer->TraceCommandOK();

			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}

		} else {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}
			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);
		}

		break;
	}
/*
	case 0x4c: // speedy: jump to address
	{
		// routine at $ff5a sends ACK

		if (fEnableHighSpeed && 
			(frame.aux1 == 0x5a) && (frame.aux2 == 0xff) ) {

			if ((ret=wrapper->SendCommandACK())) {
				fTracer->TraceCommandError(ret);
				DPRINTF("send command ACK failed");
				break;
			}
	
			fTracer->TraceCommandOK();

			if ((ret=wrapper->SendComplete())) {
				DPRINTF("send complete failed");
				break;
			}

		} else {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}
			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);
		}

		break;
	}
	case 0x41: {
		// Speedy: add new command
		// do nothing, just ACK it

		if (fEnableHighSpeed) {
			if ((ret=wrapper->SendCommandACK()) ) {
				DPRINTF("send command ACK failed");
				break;
			}

			size_t buflen = 3;
			unsigned char buf[buflen];
			const char* description = "[ add command ]";

			if ((ret=wrapper->ReceiveDataFrame(buf, buflen)) ) {
				fTracer->TraceCommandError(ret);
				DPRINTF("receive data frame failed");
				break;
			}

			fTracer->TraceCommandOK();
			fTracer->TraceDataBlock(buf, buflen, description);

			if ((ret=wrapper->SendComplete()) ) {
				DPRINTF("send complete failed");
				break;
			}
		} else {
			if (wrapper->SendCommandNAK()) {
				DPRINTF("send command NAK failed");
			}
			ret = AbstractSIOHandler::eUnsupportedCommand;
			fTracer->TraceCommandError(ret);
		}

		break;
	}
*/
	default:
		if (wrapper->SendCommandNAK()) {
			DPRINTF("send command NAK failed");
		}
		ret = AbstractSIOHandler::eUnsupportedCommand;
		fTracer->TraceCommandError(ret);
		break;
	}

	return ret;
}

bool AtrSIOHandler::IsAtrSIOHandler() const
{
	return true;
}

bool AtrSIOHandler::EnableHighSpeed(bool on)
{
	fEnableHighSpeed = on;
/*
	if (on) {
		fSpeedByte = SPEED_BYTE_57600;
	} else {
		fSpeedByte = SPEED_BYTE_19200;
	}
*/
	return true;
}
