#ifndef CURSESFRONTEND_H
#define CURSESFRONTEND_H

/*
   CursesFrontend.h - curses frontend for atariserver

   Copyright (C) 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <curses.h>
#include <panel.h>
#include "DiskManager.h"
#include "History.h"
#include "DirectoryCache.h"

class CursesFrontend {
public:
	CursesFrontend(RCPtr<DiskManager>& manager, bool useColor = true);

	virtual ~CursesFrontend();

	void HandleResize(bool redrawDriveStatusOnly = false);

	void InitWindows();

	void InitDriveStatusWindows();
	void InitTopAndBottomLines();

	// drive = eDrive[1-8]: only specified drive
	// drive = eAllDrives  : all drives
	void DisplayDriveStatus(DiskManager::EDriveNumber drive = DiskManager::eAllDrives);

	void DisplayDriveNumber(DiskManager::EDriveNumber drive);
	void DisplayDriveChangedStatus(DiskManager::EDriveNumber drive);
	void DisplayDriveProtectedStatus(DiskManager::EDriveNumber drive);
	void DisplayDriveDensity(DiskManager::EDriveNumber drive);
	void DisplayDriveSectors(DiskManager::EDriveNumber drive);
	void DisplayDriveFilename(DiskManager::EDriveNumber drive);

	// return old status
	bool ShowCursor(bool on);

	bool ShowDos2Directory(const RCPtr<DiskImage>& image, bool beQuiet = false);

	void ShowText(const char* title, const char* const* text);

	void AddLogLine(const char* str);
	void PrintLogLine(const char* format, ...)
		__attribute__ ((format (printf, 2, 3))) ;

	// CursesTracer methods
	void StartLogLine();
	void EndLogLine();
	void AddLogString(const char* str);
	void AddLogHighlightString(const char* str);
	void AddLogOKString(const char* str);
	void AddLogDebugString(const char* str);
	void AddLogWarningString(const char* str);
	void AddLogErrorString(const char* str);

	// update screen buffers
	void UpdateScreen();

	int GetCh(bool ignoreResize);

	void SetTraceLevel(int level);

	int GetTraceLevel() const { return fTraceLevel; }

	void DisplayStatusLine();

	bool ProcessQuit();
	void ProcessLoadDrive();
	void ProcessWriteDrive();
	void ProcessCreateDrive();
	void ProcessWriteProtectDrive();
	void ProcessUnprotectDrive();
	void ProcessExchangeDrives();
	void ProcessUnloadDrive();
	void ProcessSetTraceLevel();
	void ProcessSetCableType();
	void ProcessSetHighSpeed();
	void ProcessShowDirectory();
	void ProcessShowHelp();

	void AddFilenameHistory(const char* string);

	// return old status
	bool ShowAuxWindow(bool on);

	void IndicateGotSigWinCh() { fGotSigWinCh = true; }
	bool GotSigWinCh() { return fGotSigWinCh; }

	void InitTopLine();
	void SetTopLineString(const char* string);
	void SetTopLineFilename(const char* string, bool appenSlash);

	unsigned int GetScreenWidth() { return fScreenWidth; }
	unsigned int GetScreenHeight() { return fScreenHeight; }

private:
	friend class FileSelect;
	void ShowHint(const char* string);
	RCPtr<DirectoryCache>& GetDirectoryCache() { return fDirCache; }

	attr_t GetAuxColorStandard() { return fAuxColorStandard; }
	attr_t GetAuxColorSelected() { return fAuxColorSelected; }
	WINDOW* GetInputLineWindow() { return fInputLineWindow; }

private:
	void InternalProcessWriteProtectDrive(bool protect);

	bool CalculateWindowPositions(
		int& toplineStart, int& toplineHeight,
		int& drivestatusStart, int& drivestatusHeight,
		int& logtitleStart, int& logtitleHeight,
		int& logStart, int& logHeight,
		int& bottomStart, int& bottomHeight,
		int& inputStart, int& inputHeight,
		int& auxStart, int& auxHeight);

	void ClearInputLine();

	bool IsAbortChar(int ch);
	void AbortInput();

	DiskManager::EDriveNumber InputDriveNumber(bool allowAll = false);
	DiskManager::EDriveNumber InputUsedDriveNumber(bool allowAll = false);
	DiskManager::EDriveNumber InputUnusedDriveNumber(bool allowAll = false);

	enum EYesNo {
		eYN_Abort = -1,
		eYN_No = 0,
		eYN_Yes = 1
	};
	EYesNo InputYesNo();

	void ShowStandardHint();
	void ShowYesNoHint();
	void ShowPagerHint();
	void ShowFileInputHint();
	void ShowTraceLevelHint();
	void ShowCreateDriveHint();
	void ShowImageSizeHint();
	void ShowHighSpeedHint();

	enum EDriveInputHintType {
		eDriveInputStandard,
		eDriveInputAll,
		eDriveInputAllChanged
	};
	void ShowDriveInputHint(EDriveInputHintType type);

	void ShowDriveNumber(DiskManager::EDriveNumber d, bool allChanged=false);
	void ShowYesNo(EYesNo yn);

	enum { eFileSelectKey = 6 }; // ^F

	unsigned int fScreenWidth, fScreenHeight;

	// permanent windows
	WINDOW* fTopLineWindow;
	WINDOW* fDriveStatusWindow;
	WINDOW* fStatusLineWindow;
	WINDOW* fLogWindow;
	WINDOW* fBottomLineWindow;
	WINDOW* fInputLineWindow;
	WINDOW* fAuxWindow;

	PANEL* fTopLinePanel;
	PANEL* fDriveStatusPanel;
	PANEL* fStatusLinePanel;
	PANEL* fLogPanel;
	PANEL* fBottomLinePanel;
	PANEL* fInputLinePanel;
	PANEL* fAuxPanel;

	bool fFirstLogLine;

	RCPtr<DiskManager> fDiskManager;

	// drive display positions
	unsigned int fDriveYStart;
	unsigned int fDriveNoX;
	unsigned int fDriveChangedX;
	unsigned int fDriveProtectedX;
	unsigned int fDriveDensityX;
	unsigned int fDriveSectorsX;
	unsigned int fDriveFilenameX;
	unsigned int fDriveFilenameLength;

	// log window colors
	attr_t fLogColorStandard;
	attr_t fLogColorHighlight;
	attr_t fLogColorOK;
	attr_t fLogColorDebug;
	attr_t fLogColorWarning;
	attr_t fLogColorError;

	// drive status window colors
	attr_t fDriveColorStandard;
	attr_t fDriveColorWritable;
	attr_t fDriveColorWriteProtected;
	attr_t fDriveColorChanged;

	attr_t fStatusColorStandard;

	attr_t fAuxColorStandard;
	attr_t fAuxColorSelected;

	int fTraceLevel;

	History fFilenameHistory;
	History fImageSizeHistory;
	RCPtr<DirectoryCache> fDirCache;

	bool fCursorStatus;
	bool fAuxWindowStatus;

	bool fGotSigWinCh;
};

#endif
