/*
   DiskImage.cpp - common base class for handling images

   Copyright (C) 2002 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <string.h>

#include "DiskImage.h"
#include "SIOTracer.h"
#include "AtariDebug.h"

DiskImage::DiskImage()
	: fFilename(0),
	  fWriteProtect(false),
	  fChanged(false)
{
}

DiskImage::~DiskImage()
{
	if (fFilename) {
		free(fFilename);
		fFilename=0;
	}
}

const char* DiskImage::GetFilename() const
{
	return fFilename;
}

void DiskImage::SetFilename(const char* filename)
{
	if (fFilename) {
		free(fFilename);
	}
	if (filename) {
		fFilename=strdup(filename);
	} else {
		fFilename=0;
	}
}

bool DiskImage::WriteBackImageToFile() const
{
	if (GetFilename()) {
		return WriteImageToFile(GetFilename());
	} else {
		return false;
	}
}

bool DiskImage::IsAtrImage() const
{
	return false;
}

bool DiskImage::IsAtpImage() const
{
	return false;
}

Dos2Dir::Dos2Dir()
	: fFileCount(0), fFreeSectors(0)
{
	for (unsigned int i=0;i<eMaxFiles;i++) {
		fFiles[i] = NULL;
	}
}

Dos2Dir::~Dos2Dir()
{
	for (unsigned int i=0;i<eMaxFiles;i++) {
		if (fFiles[i]) {
			delete[] fFiles[i];
		}
	}
}

RCPtr<Dos2Dir> DiskImage::GetDos2Directory(bool beQuiet) const
{
	RCPtr<Dos2Dir> dir = new Dos2Dir;

	unsigned int sector = 361;
	unsigned int fileno;
	unsigned char secbuf[256];
	unsigned int seclen;
	bool done = false;
	unsigned int i;
	unsigned int entriesPerSector = 8;
	bool isBigImage = false;

	seclen = GetSectorLength();
	
	if (seclen != 128 && seclen != 256) {
		DPRINTF("illegal sector length %d", seclen);
		return 0;
	}
	if (GetNumberOfSectors() < 368) {
		if (!beQuiet) {
			ERROR("cannot read directory - image contains less than 368 sectors");
		}
		return 0;
	}

	if (!ReadSector(360, secbuf, seclen)) {
		DPRINTF("error reading VTOC sector 360");
		return 0;
	}
	dir->fFreeSectors = secbuf[3] + (secbuf[4] << 8);

	if ( (seclen == 256) && (secbuf[0] >= 3) ) {
		isBigImage = true;
	}

	while (!done) {
		if (!ReadSector(sector, secbuf, seclen)) {
			DPRINTF("error reading directory sector %d", sector);
			return dir;
		}
		if ( isBigImage && (sector == 361) ) {
			if (secbuf[128] != 0) {
				entriesPerSector = 16;
			}
		}
		fileno = 0;
		while (fileno < entriesPerSector) {
			unsigned char stat = secbuf[fileno*16];
			if (stat == 0) {
				done = true;
				break;
			}
			if ( ((stat & 0xc3) == 0x42) || ((stat & 0xc3) == 0x03) ) {
				char* str = new char[Dos2Dir::eFileWidth + 1];
				dir->fFiles[dir->fFileCount++] = str;

				if (stat & 0x20) {
					str[0] = '*';
				} else {
					str[0] = ' ';
				}
				str[1] = ' ';
				
				for (i=0;i<11;i++) {
					char tmp = secbuf[fileno*16+5+i] & 0x7f;
					if (tmp < 32 || tmp > 126) {
						tmp='.';
					}
					str[i+2] = tmp;
				}
				str[13] = 0;
				unsigned int sec = secbuf[fileno*16+1] + (secbuf[fileno*16+2] << 8);
				if (sec < 10000) {
					snprintf(str+13, Dos2Dir::eFileWidth-12, " %04d  ", sec);
				} else {
					snprintf(str+13, Dos2Dir::eFileWidth-12, " %d ", sec);
				}
			}
			fileno++;
		}
		sector++;
		if (sector == 369) {
			done = true;
		}
	}
	return dir;
}
