/*
   DiskManager.cpp - collection of helper routines to manage disk images
   and load/unload them into the SIO server (SIOManager)

   Copyright (C) 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <limits.h>
#include <string.h>

#include "DiskManager.h"
#include "AtrMemoryImage.h"
#include "AtrSIOHandler.h"
#include "AtpImage.h"
#include "AtpSIOHandler.h"
#include "AtpUtils.h"
#include "Error.h"

#include "AtariDebug.h"

DiskManager::DiskManager()
	: fSIOWrapper(new SIOWrapper),
	  fSIOManager(new SIOManager(fSIOWrapper))
{
	if (!SetSioServerMode(SIOWrapper::eCommandLine_RI)) {
		throw ErrorObject("unable to activate SIO server mode");
	}

	SetHighSpeedMode(eHighSpeedOn);
}

DiskManager::~DiskManager()
{
}

bool DiskManager::SetSioServerMode(SIOWrapper::ESIOServerCommandLine cmdLine)
{
	if (fSIOWrapper->SetSIOServerMode(cmdLine)) {
		DPRINTF("cannot init SIO server mode!");
		return false;
	}
	fCableType = cmdLine;
	return true;
}

bool DiskManager::DriveInUse(EDriveNumber driveno) const
{
	if (!DriveNumberOK(driveno)) {
		return 1;
	}
	return fSIOManager->GetConstHandler(eSIODriveBase+driveno).IsNotNull();
}

RCPtr<DiskImage> DiskManager::LoadDiskImage(const char* filename, bool beQuiet)
{
	RCPtr<DiskImage> image;

	char absPath[PATH_MAX+1];
	if (realpath(filename,absPath) == 0) {
		if (!beQuiet) {
			ERROR("cannot find \"%s\"", filename);
		}
		return image;
	}

	int len = strlen(absPath);
	if ( (len > 4 && strcasecmp(absPath+len-4,".atp") == 0) ||
	     (len > 7 && strcasecmp(absPath+len-7,".atp.gz") == 0)) {
		RCPtr<AtpImage> img(new AtpImage);
		if (img->ReadImageFromFile(absPath, beQuiet)) {
			image = img;
		}
	} else {
		RCPtr<AtrMemoryImage> img(new AtrMemoryImage);
		if (img->ReadImageFromFile(absPath, beQuiet)) {
			image = img;
		}
	}

	if (image.IsNotNull()) {
		image->SetFilename(absPath);
	}

	return image;
}

bool DiskManager::LoadDiskImage(EDriveNumber driveno, const char* filename, bool beQuiet)
{
	if (!DriveNumberOK(driveno)) {
		return false;
	}

	if (DriveInUse(driveno)) {
		if (!beQuiet) {
			ERROR("already loaded image into D%d: - unload first",driveno);
		}
		return false;
	}

	RCPtr<DiskImage> image = LoadDiskImage(filename, beQuiet);

	if (image.IsNull()) {
		return false;
	}

	RCPtr<AbstractSIOHandler> handler;

	if (image->IsAtpImage()) {
		handler = new AtpSIOHandler(RCPtrStaticCast<AtpImage>(image));
	} else if (image->IsAtrImage()) {
		handler = new AtrSIOHandler(RCPtrStaticCast<AtrImage>(image));
		handler->EnableHighSpeed(fUseHighSpeed);
	} else {
		return false;
	}

	if (!fSIOManager->RegisterHandler(eSIODriveBase+driveno, handler)) {
		if (!beQuiet) {
			DPRINTF("cannot register device %d!",eSIODriveBase+driveno);
		}
		return false;
	}

	if (!beQuiet) {
		LOG("loaded D%d: from \"%s\"", driveno, image->GetFilename());
	}
	return true;
}

bool DiskManager::CreateAtrMemoryImage(EDriveNumber driveno, EDiskFormat format)
{
	if (!DriveNumberOK(driveno)) {
		return false;
	}

	if (DriveInUse(driveno)) {
		return false;
	}

	RCPtr<AtrMemoryImage> img(new AtrMemoryImage);
	img->CreateImage(format);
	img->SetChanged(false);

	RCPtr<AtrSIOHandler> handler(new AtrSIOHandler(img));
	handler->EnableHighSpeed(fUseHighSpeed);

	if (!fSIOManager->RegisterHandler(eSIODriveBase+driveno, handler)) {
		DPRINTF("cannot register device %d!",eSIODriveBase+driveno);
		return false;
	}

	return true;
}

bool DiskManager::CreateAtrMemoryImage(EDriveNumber driveno, ESectorLength density, size_t sectors)
{
	if (!DriveNumberOK(driveno)) {
		return false;
	}

	if (DriveInUse(driveno)) {
		return false;
	}

	RCPtr<AtrMemoryImage> img(new AtrMemoryImage);
	img->CreateImage(density, sectors);
	img->SetChanged(false);

	RCPtr<AtrSIOHandler> handler(new AtrSIOHandler(img));
	handler->EnableHighSpeed(fUseHighSpeed);

	if (!fSIOManager->RegisterHandler(eSIODriveBase+driveno, handler)) {
		DPRINTF("cannot register device %d!",eSIODriveBase+driveno);
		return false;
	}

	return true;
}

bool DiskManager::UnloadDiskImage(EDriveNumber driveno)
{
	int min, max;

	if (driveno == eAllDrives) {
		min = eMinDriveNumber;
		max = eMaxDriveNumber;
	} else {
		if (!DriveNumberOK(driveno)) {
			return false;
		}
		min = driveno;
		max = driveno;
	}

	for (int i=min; i<=max;i++) {
		if (DriveInUse(EDriveNumber(i))) {
			fSIOManager->UnregisterHandler(eSIODriveBase+i);
		}
	}
	return true;
}

bool DiskManager::SetWriteProtectImage(EDriveNumber driveno, bool on)
{
	int min, max;

	if (driveno == eAllDrives) {
		min = eMinDriveNumber;
		max = eMaxDriveNumber;
	} else {
		if (!DriveNumberOK(driveno)) {
			return false;
		}
		if (!DriveInUse(driveno)) {
			DPRINTF("drive D%d: is empty",driveno);
			return false;
		}
		min = driveno;
		max = driveno;
	}

	for (int i=min; i<=max;i++) {
		if (DriveInUse(EDriveNumber(i))) {
			RCPtr<AbstractSIOHandler> absHandler = fSIOManager->GetHandler(eSIODriveBase+i);
			RCPtr<DiskImage> image = absHandler->GetDiskImage();

			if (image) {
				image->SetWriteProtect(on);
			}
		}
	}
	return true;
}

bool DiskManager::WriteBackImagesIfChanged()
{
	int i;
	int ok=true;
	for (i=eMinDriveNumber;i<=eMaxDriveNumber;i++) {
		if (DriveInUse(EDriveNumber(i))) {
			RCPtr<AbstractSIOHandler> absHandler = fSIOManager->GetHandler(eSIODriveBase+i);
			RCPtr<DiskImage> diskImage = absHandler->GetDiskImage();
			if (diskImage->GetFilename() && diskImage->Changed()) {
				if (!diskImage->WriteBackImageToFile()) {
					LOG("ERROR writing D%d: to \"%s\"", i, diskImage->GetFilename());
					ok = false;
				} else {
					LOG("wrote D%d: to \"%s\"", i, diskImage->GetFilename());
				}
			}
		}
	}
	return ok;
}

bool DiskManager::WriteDiskImage(EDriveNumber driveno, const char* filename)
{
	if (!DriveNumberOK(driveno)) {
		return false;
	}
	if (!DriveInUse(driveno)) {
		DPRINTF("drive D%d: is empty",driveno);
		return false;
	}

	if (!filename) {
		DPRINTF("filename is NULL!");
		return false;
	}
	
	RCPtr<AbstractSIOHandler> absHandler = fSIOManager->GetHandler(eSIODriveBase+driveno);

	if (absHandler) {
		RCPtr<DiskImage> diskImage = absHandler->GetDiskImage();

		int len = strlen(filename);

		bool ok;
		if ( (len > 4 && strcasecmp(filename+len-4,".atp") == 0) ||
		     (len > 7 && strcasecmp(filename+len-7,".atp.gz") == 0)) {
			if (diskImage->IsAtrImage()) {
				ok = false;
				RCPtr<FileIO> fileio;
				RCPtr<AtrImage> atrImage = RCPtrStaticCast<AtrImage>(diskImage);
				RCPtr<AtpImage> atpImage = AtpUtils::CreateAtpImageFromAtrImage(atrImage);
				if (!atpImage) {
					goto error;
				}
				if (len > 7 && strcasecmp(filename+len-7,".atp.gz")) {
#ifdef USE_ZLIB
					fileio = new GZFileIO();
#else
					ERROR("cannot write compressed file - zlib support disabled at compiletime!");
					goto error;
#endif
				} else {
					fileio = new StdFileIO();
				}
				ok = atpImage->WriteImageToFile(filename);
				if (ok) {
					diskImage->SetChanged(false);
				}
			} else {
				ok=diskImage->WriteImageToFile(filename);
			}
			if (ok && !diskImage->IsAtpImage()) {
				LOG("changed image type to ATP - reloading drive D%d:",driveno);
				UnloadDiskImage(driveno);
				ok = LoadDiskImage(driveno, filename);
			}
		} else {
			if (diskImage->IsAtpImage()) {
				ERROR("cannot convert ATP image into other image types!");
				return false;
			}
			ok=diskImage->WriteImageToFile(filename);
		}

error:
		if (!ok) {
			return false;
		} else {
			char absPath[PATH_MAX+1];
			if (realpath(filename,absPath) == 0) {
				DPRINTF("get realpath failed");
				return false;
			}
			diskImage->SetFilename(absPath);
			LOG("wrote D%d: to \"%s\"", driveno, diskImage->GetFilename());
		}
	} else {
		DPRINTF("internal error - can't get SIO handler!");
		return false;
	}
	return true;
}

bool DiskManager::ExchangeDrives(EDriveNumber drive1, EDriveNumber drive2)
{
	if (!DriveNumberOK(drive1) || !DriveNumberOK(drive2)) {
		return false;
	}
	if (drive1 == drive2) {
		DPRINTF("exchanging a drive with itself is not allowed!");
		return false;
	}
	RCPtr<AbstractSIOHandler> h1 (fSIOManager->GetHandler(eSIODriveBase+drive1));
	RCPtr<AbstractSIOHandler> h2 (fSIOManager->GetHandler(eSIODriveBase+drive2));

	fSIOManager->UnregisterHandler(eSIODriveBase+drive1);
	fSIOManager->UnregisterHandler(eSIODriveBase+drive2);

	fSIOManager->RegisterHandler(eSIODriveBase+drive1,h2);
	fSIOManager->RegisterHandler(eSIODriveBase+drive2,h1);
	return true;
}

const char* DiskManager::GetImageFilename(EDriveNumber driveno) const
{
	if (!DriveNumberOK(driveno)) {
		return 0;
	}
	if (DriveInUse(driveno)) {
		RCPtr<const AbstractSIOHandler> absHandler = fSIOManager->GetConstHandler(eSIODriveBase+driveno);

		RCPtr<const DiskImage> image = absHandler->GetConstDiskImage();

		if (image) {
			return image->GetFilename();
		}
	}
	return 0;
}

bool DiskManager::DriveIsWriteProtected(EDriveNumber driveno) const
{
	if (!DriveNumberOK(driveno)) {
		return false;
	}
	if (DriveInUse(driveno)) {
		RCPtr<const AbstractSIOHandler> absHandler = fSIOManager->GetConstHandler(eSIODriveBase+driveno);

		RCPtr<const DiskImage> image = absHandler->GetConstDiskImage();

		if (image) {
			return image->IsWriteProtected();
		}
	}
	return false;
}

bool DiskManager::DriveIsChanged(EDriveNumber driveno) const
{
	int min, max;

	if (driveno == eAllDrives) {
		min = eMinDriveNumber;
		max = eMaxDriveNumber;
	} else {
		if (!DriveNumberOK(driveno)) {
			return false;
		}
		if (!DriveInUse(driveno)) {
			DPRINTF("drive D%d: is empty",driveno);
			return false;
		}
		min = driveno;
		max = driveno;
	}
	for (int i=min; i<=max; i++) {
		if (DriveInUse( (EDriveNumber) i ) ) {
			RCPtr<const DiskImage> img = fSIOManager->GetConstHandler(eSIODriveBase+i)->GetConstDiskImage();
			if (img.IsNull()) {
				DPRINTF("got null image in DriveIsChanged()");
			} else {
				if (img->Changed()) {
					return true;
				}
			}
		}
	}
	return false;
}

size_t DiskManager::GetDriveImageSize(EDriveNumber driveno) const
{
	if (!DriveNumberOK(driveno)) {
		return 0;
	}
	if (DriveInUse(driveno)) {
		RCPtr<const AbstractSIOHandler> absHandler = fSIOManager->GetConstHandler(eSIODriveBase+driveno);

		if (absHandler->IsAtrSIOHandler()) {
			RCPtr<const AtrSIOHandler> handler = RCPtrStaticCast<const AtrSIOHandler>(absHandler);
			return handler->GetConstAtrImage()->GetImageSize();
		}
	}
	return 0;
}

int DiskManager::DoServing(int otherReadPollDevice)
{
	return fSIOManager->DoServing(otherReadPollDevice);
}

bool DiskManager::DriveNumberOK(EDriveNumber driveno) const
{
	if (driveno < eMinDriveNumber || driveno > eMaxDriveNumber ) {
		DPRINTF("illegal drive number %d",driveno);
		return false;
	}
	return true;
}

bool DiskManager::SetHighSpeedMode(EHighSpeedMode mode)
{
	fHighSpeedMode = mode;
	switch (mode) {
	case eHighSpeedOff:
		if (fSIOWrapper->SetBaudrate(ATARISIO_STANDARD_BAUDRATE)) {
			return false;
		}
		if (fSIOWrapper->SetAutobaud(0)) {
			return false;
		}
		fUseHighSpeed = false;
		break;
	case eHighSpeedOn:
		if (fSIOWrapper->SetBaudrate(ATARISIO_HIGHSPEED_BAUDRATE)) {
			return false;
		}
		if (fSIOWrapper->SetAutobaud(1)) {
			return false;
		}
		if (fSIOWrapper->SetHighSpeedPause(0)) {
			return false;
		}
		fUseHighSpeed = true;
		break;
	case eHighSpeedWithPause:
		if (fSIOWrapper->SetBaudrate(ATARISIO_HIGHSPEED_BAUDRATE)) {
			return false;
		}
		if (fSIOWrapper->SetAutobaud(1)) {
			return false;
		}
		if (fSIOWrapper->SetHighSpeedPause(1)) {
			return false;
		}
		fUseHighSpeed = true;
		break;
	}

	for (int i=eMinDriveNumber; i<=eMaxDriveNumber; i++) {
		if (DriveInUse(EDriveNumber(i))) {
			fSIOManager->GetHandler(eSIODriveBase+i)->EnableHighSpeed(fUseHighSpeed);
		}
	}
	return true;
}

RCPtr<AtrImage> DiskManager::GetAtrImage(EDriveNumber driveno)
{
	if (!DriveNumberOK(driveno)) {
		return RCPtr<AtrImage>();
	}
	RCPtr<AbstractSIOHandler> handler(fSIOManager->GetHandler(driveno+0x30));
	if (handler && handler->IsAtrSIOHandler()) {
		RCPtr<AtrSIOHandler> atrHandler = RCPtrStaticCast<AtrSIOHandler>(handler);
		return atrHandler->GetAtrImage();
	}
	return RCPtr<AtrImage>();
}

RCPtr<const AtrImage> DiskManager::GetConstAtrImage(EDriveNumber driveno) const
{
	if (!DriveNumberOK(driveno)) {
		return RCPtr<AtrImage>();
	}
	RCPtr<const AbstractSIOHandler> handler(fSIOManager->GetConstHandler(driveno+0x30));
	if (handler && handler->IsAtrSIOHandler()) {
		RCPtr<const AtrSIOHandler> atrHandler(RCPtrStaticCast<const AtrSIOHandler> (handler));
		return atrHandler->GetConstAtrImage();
	}
	return RCPtr<AtrImage>();
}

RCPtr<DiskImage> DiskManager::GetDiskImage(EDriveNumber driveno)
{
	if (!DriveNumberOK(driveno)) {
		return RCPtr<DiskImage>();
	}
	RCPtr<AbstractSIOHandler> handler(fSIOManager->GetHandler(driveno+0x30));
	if (handler) {
		return handler->GetDiskImage();
	}
	return RCPtr<DiskImage>();
}

RCPtr<const DiskImage> DiskManager::GetConstDiskImage(EDriveNumber driveno) const
{
	if (!DriveNumberOK(driveno)) {
		return RCPtr<DiskImage>();
	}
	RCPtr<const AbstractSIOHandler> handler(fSIOManager->GetConstHandler(driveno+0x30));
	if (handler) {
		return handler->GetConstDiskImage();
	}
	return RCPtr<DiskImage>();
}

bool DiskManager::CheckForChangedImages()
{
	return DriveIsChanged(eAllDrives);
}
